@@ -489,7 +489,7 @@ class APTForeignCheck(Check):
         super().__init__(_("Foreign packages"), _("Looking for foreign packages..."), callback)
 
     def do_run(self):
-        orphans, self.foreigns = get_foreign_packages(find_orphans=False, find_downgradable_packages=True)
+        orphans, orphan_t64_names, self.foreigns = get_foreign_packages(find_orphans=False, find_downgradable_packages=True)
         if len(self.foreigns) > 0:
             self.result = RESULT_ERROR
             self.message = _("The following packages need to be downgraded back to official versions:")
@@ -521,7 +521,7 @@ class APTOrphanCheck(Check):
     def do_run(self):
         self.pre_upgrade_orphans.clear()
 
-        orphans, foreigns = get_foreign_packages(find_orphans=True, find_downgradable_packages=False)
+        orphans, orphan_t64_names, foreigns = get_foreign_packages(find_orphans=True, find_downgradable_packages=False)
         if len(orphans) > 0:
             for orphan in orphans:
                 pkg, version = orphan
@@ -560,7 +560,7 @@ class APTRemoveOrphansCheck(Check):
 
     def do_run(self):
         orphan_pkgs = []
-        orphans, foreigns = get_foreign_packages(find_orphans=True, find_downgradable_packages=False)
+        orphans, orphan_t64_names, foreigns = get_foreign_packages(find_orphans=True, find_downgradable_packages=False)
         if len(orphans) > 0:
             for orphan in orphans:
                 pkg, version = orphan
@@ -583,6 +583,13 @@ class APTRemoveOrphansCheck(Check):
                 # The return code indicates a failure if some packages were not found, so ignore it.
                 run_command('%s purge --yes %s' % (APT_GET, removal))
 
+        if len(orphan_t64_names) > 0:
+            # Upgrade t64 orphans to t64 versions
+            print_output("Migrating t64 orphans")
+            for name in orphan_t64_names:
+                # The return code indicates a failure if some packages were not found, so ignore it.
+                run_command('%s install --yes %s' % (APT_GET, name))
+
 # Switch to the target repositories
 class UpdateReposCheck(Check):
 
@@ -942,14 +949,26 @@ class PostUpgradeCheck(Check):
 
         # Install new packages
         print_output("Installing new packages")
-        if len(PACKAGES_ADDITIONS) > 0:
-            if not run_command('%s install --yes --no-install-recommends %s' % (APT_GET, " ".join(PACKAGES_ADDITIONS))):
+        additions = PACKAGES_ADDITIONS
+        if os.getenv("XDG_CURRENT_DESKTOP") != None:
+            desktop = os.getenv("XDG_CURRENT_DESKTOP").upper().replace("X-", "")
+            if desktop == "CINNAMON":
+                print_output("Installing new packages for Cinnamon as well")
+                additions += PACKAGES_ADDITIONS_CINNAMON
+            elif desktop == "MATE":
+                print_output("Installing new packages for MATE as well")
+                additions += PACKAGES_ADDITIONS_MATE
+            elif desktop == "XFCE":
+                print_output("Installing new packages for Xfce as well")
+                additions += PACKAGES_ADDITIONS_XFCE
+        if len(additions) > 0:
+            if not run_command('%s install --yes --no-install-recommends %s' % (APT_GET, " ".join(additions))):
                 self.result = RESULT_ERROR
                 self.message = _("The following packages could not be installed:")
                 table_list = TableList([""])
                 table_list.show_column_names = False
                 cache = apt.Cache()
-                for name in PACKAGES_ADDITIONS:
+                for name in additions:
                     if name in cache:
                         pkg = cache[name]
                         if not pkg.is_installed:
